import os
import time

from datetime import datetime

class Output():
    def __init__(self) -> None:
        self.startTime = time.time()

    def printHeader(self):
        print("################################################################################")
        print("")
        print("dtgn (datagen) by 5f0")
        print("Creates folder structures and puts data in it")
        print("")
        print("Current working directory: " + os.getcwd())
        print("")
        print("Datetime: " + datetime.now().strftime("%d/%m/%Y %H:%M:%S"))
        print("")
        print("################################################################################")
        print("")

    def printFolderFileInfo(self, args, rootPath):
        print("Mode: " + args.mode)
        print("Create folder structure under " + rootPath)
        print("Create folder depth of " + str(args.depth))
        print("Create " + str(args.fileCount) + " files per folder")
        if(args.mode == "fixed"):
            print("Size per file: " + str(args.fileMinSize) + " Bytes")
        elif(args.mode == "random"):
            print("Size per file: between " + str(args.fileMinSize) + " and " + str(args.fileMaxSize) + " Bytes")
        print("")
        print("################################################################################")
        print("")

    def printSizeInfo(self, totalSize):
        print("Total File Size created: " + str(totalSize) + " Bytes")
        kb = totalSize/1000
        print("                         " + str(kb) + " KB")
        mb = kb/1000
        print("                         " + str(mb) + " MB")
        gb = mb/1000
        print("                         " + str(gb) + " GB")

    def printExecutionTime(self):
        end = time.time()
        print("")
        print("################################################################################")
        print("")
        print("Execution Time: " + str(end-self.startTime)[0:8] + " sec")
        print("")


import os
import sys
import random
import shutil
import argparse

from pathlib import Path

from dtgn.args.Args import Args
from dtgn.output.Output import Output

def main(args_=None):
    """The main routine."""
    if args_ is None:
        args_ = sys.argv[1:]

    parser = argparse.ArgumentParser()
    Args.addArguments(parser)
    args = parser.parse_args()

    # Creates Output instance for printing header and footer of console output
    out = Output()
    out.printHeader()

    # Create root path
    rootPath = os.path.join(args.path, args.rootName)

    if(args.purge):
        shutil.rmtree(rootPath)
        print(rootPath + " purged!")
    else:
        # Print argument configuration
        out.printFolderFileInfo(args, rootPath)

        # Create root dir
        Path(rootPath).mkdir(parents=True, exist_ok=True)

        # Create the folders through the given depth
        totalSize = 0
        for i in range(1,args.depth+1):
            folderName = "folder" + str(i)
            f = os.path.join(rootPath, folderName)
            Path(f).mkdir(parents=True, exist_ok=True)
            # Create files
            for i in range(1, args.fileCount+1):
                filePath = os.path.join(f, folderName + "-" + str(i) + ".bin")
                with open(filePath, 'wb') as file:
                    size = 0
                    if(args.mode == "fixed"):
                        size = args.fileMinSize
                        file.write(bytearray(os.urandom(size)))
                    elif(args.mode == "random"):
                        size = random.randint(args.fileMinSize, args.fileMaxSize)
                        file.write(bytearray(os.urandom(size)))
                    totalSize = totalSize + size
        
            rootPath = f
        out.printSizeInfo(totalSize)

    out.printExecutionTime()


if __name__ == "__main__":
    sys.exit(main())



import argparse

class Args():
    # parser: argparse instance
    def addArguments(parser):
        parser.add_argument("--mode", "-m", choices=["fixed","random"], default="fixed", help="Data Creation Mode: fixed: all files will be created the same size. Size is taken fom fileMinSize argument. random: the files created will have a random size between fileMinSize and fileMaxSize argument.")
        parser.add_argument("--path", "-p", type=str, required=True, help="Path under which the folder structure shall be located")
        parser.add_argument("--rootName", "-r", type=str, default="dtgn", help="The name of the root folder created")
        parser.add_argument("--depth", "-d", type=int, default=1, help="The depth of the folder structure")
        parser.add_argument("--fileCount", "-f", type=int, default=1, help="Nr. of files per folder depth")
        parser.add_argument("--fileMinSize", "-g", type=int, default=2000, help="The minium file size in bytes")
        parser.add_argument("--fileMaxSize", "-i", type=int, default=2000, help="The maximum file size in bytes")
        parser.add_argument("--purge", "-u", action=argparse.BooleanOptionalAction, help="If true, all files in path+rootName will be deleted. No folder structure is created. Use it carefully.")